#include "NewtBody.hpp"

// Fonction callback static pour grer les forces
static void ForceAndTorqueCallback (const NewtonBody * nBody)
{
   float fMasse;
   CVector vInertie;

   NewtonBodyGetMassMatrix (nBody, &fMasse, &vInertie.x, &vInertie.y, &vInertie.z);

   CVector vForce (0.0f, -fMasse * 9.81, 0.0f);

   NewtonBodyAddForce (nBody, &vForce.x);
}

// Fonction callback static pour les transformations
static void TransformCallback (const NewtonBody * nBody, const float * matrix)
{
	// On rcupre le corps et le modle
	NewtBody * pBody = (NewtBody *)NewtonBodyGetUserData (nBody);
	CModel & model = pBody->GetModel ();

	model.SetPosition (matrix); // On met  jour la position du modle 3D
}

// Constructeur
NewtBody::NewtBody (const NewtCollision & nCollision, CModel * model, const CVector & position)
	: _nCollision (nCollision), _masse (0)
{
	_model = *model; // On copie model dans notre objet _model

	_nWorld = NewtWorld::GetInstance (); // On rcupre le pointeur vers l'objet NewtWorld
	
	// On cre notre corps avec la primitive de collision passe en paramtre
	_nBody = NewtonCreateBody (_nWorld->GetNewtonWorld(), _nCollision.GetCollision());
	
	NewtonBodySetUserData (_nBody, this); // On stocke l'objet

	Matrix myMatrix;
	myMatrix.LoadIdentity ();
	
	myMatrix._matrix [12] = position.x;
	myMatrix._matrix [13] = position.y;
	myMatrix._matrix [14] = position.z;

	_model.SetPosition (&myMatrix._matrix [0]); // On assigne au modle 3D sa position,
																// qui est la mme que celle de l'objet Newton

	NewtonBodySetMatrix (_nBody, &myMatrix._matrix [0]);
}

// Destructeur
NewtBody::~NewtBody ()
{
	NewtDestroyBody (); // On dtruit le corps
}

// Dessine le modle 3D
void NewtBody::DrawModel ()
{
	_model.Dessiner (); // On dessine l'objet. On remarque qu'avec le callback de transformation,
								// on a plus besoin d'appeler des fonctions OpenGL ici (multiplier la matrice...)
}

// Dtruit un corps
void NewtBody::NewtDestroyBody () const
{
	NewtonDestroyBody (_nWorld->GetNewtonWorld(), _nBody);
}

// Assigne une masse et des valeurs d'inertie
void NewtBody::NewtBodySetMassMatrix (const float masse)
{
	_masse = masse;

	NewtonBodySetMassMatrix (_nBody, _masse, 10 * _masse, 10 * _masse, 10 * _masse);
}

// Par dfaut, le centre de masse (ou centre d'inertie), est situ au point c(0,0,0).
// Ce rglage est idal lorsqu'il s'agit d'une primitive de collision simple comme une
// bote ou une sphre. Toutefois pour un objet aux formes tarabiscotes, le centre de
// masse est diffrent. Cette fonction doit donc tre appele pour un objet convex hull
// ou compound
void NewtBody::NewtBodySetCentreOfMass ()
{
	CVector origine; // Coordonnes du centre de masse
	CVector inertie; // Valeurs d'inertie
	
	// On appelle la fonction NewtonConvexCollisionCalculateInertialMatrix pour
	// calculer le centre d'inertie de la primitive de collision
	_nCollision.NewtConvexCollisionCalculateInertialMatrix (inertie, origine);
	
	// Puis on appelle la fonction NewtonBodySetCentreOfMass
	NewtonBodySetCentreOfMass (_nBody, &origine.x);
}

// Active ou dsactive le "continuous collision mode". Ce mode, dsactiv par dfaut,
// permet d'viter qu'un corps tant soumis  des forces trs fortes (et donc se
// dplacant trs vite), ne pntre dans un autre corps. A noter qu'un objet ayant
// ce mode activ doit avoir un matriel dont le continuous collision mode est galement
// activ
void NewtBody::NewtBodySetContinuousCollisionMode (unsigned int etat) const
{
	// Etat ne peut prendre que deux valeurs : 0 ou 1
	if (etat < 0 || etat > 1)
		etat = 0;

	NewtonBodySetContinuousCollisionMode (_nBody, etat);
}

// Assigne un callback
void NewtBody::NewtBodySetForceAndTorqueCallback () const
{
	NewtonBodySetForceAndTorqueCallback (_nBody, ForceAndTorqueCallback);
}

// Assigne un callback de transformation
void NewtBody::NewtBodySetTransformCallback () const
{
	NewtonBodySetTransformCallback (_nBody, TransformCallback);
}

// Applique un matrial au corps
void NewtBody::NewtBodySetMaterialGroupID (const int matID) const
{
	NewtonBodySetMaterialGroupID (_nBody, matID);
}

// Retourne l'identifiant du matriel associ au corps
int NewtBody::NewtBodyGetMaterialGroupID () const
{
	return (NewtonBodyGetMaterialGroupID (_nBody));
}