#include "NewtWorld.hpp"

NewtWorld * NewtWorld::_instance = NULL; // Initialisation de l'instance  NULL

// Deux fonctions, dclares static (elles ne pourront pas tre utilises au-del de
// ce fichier donc), ncessaire au dboggage
static void DebugShowGeometryCollision (const NewtonBody* body, int vertexCount, const dFloat* faceVertec, int id);
static void DebugShowBodyCollision (const NewtonBody* body);


// Constructeur de la classe NewtWorld. Il se charge tout simplement de crer notre
// monde Newton via la fonction NewtonCreate
NewtWorld::NewtWorld ()
	: debugCount (0)
{
	_nWorld = NewtonCreate (NULL, NULL); // On cre notre monde Newton
}

// Destructeur de la classe NewtWorld.
NewtWorld::~NewtWorld ()
{
	// On commence par dtruire tous les corps. En thorie, ils auront dj t dtruits
	// lors de la destruction des objets NewtBody, mais on sait jamais :)
	NewtonDestroyAllBodies (_nWorld);

	// On dtruit aussi tous les matriaux
	NewtonMaterialDestroyAllGroupID (_nWorld);

	NewtonDestroy (_nWorld); // On dtruit le monde Newton
}

// Retourne l'unique instance de la classe
NewtWorld * NewtWorld::GetInstance ()
{
	// On cre l'objet si _instance est  NULL - donc non initialise -
	if (_instance == NULL)
	   _instance = new NewtWorld;

	return _instance; // Puis on retourne l'instance
}

// Dtruit l'unique instance de la classe
void NewtWorld::Destroy ()
{
	delete _instance, _instance = NULL;
}

/// -----------------------------------------------------------------------------
/// FONCTIONS RELATIVES A NEWTON GAME DYNAMICS
/// -----------------------------------------------------------------------------
// Rgle la prcision du solver
void NewtWorld::NewtSetSolverModel (SOLVER_MODEL solverModel) const
{
	// Rien de bien compliqu, on appel la fonction NewtonSetSolverModel correspondante
	NewtonSetSolverModel (_nWorld, solverModel);
}

// Rgle la prcision des frictions
void NewtWorld::NewtSetFrictionModel (FRICTION_MODEL frictionModel) const
{
	// Pareil :)
	NewtonSetFrictionModel (_nWorld, frictionModel);
}

// Rgle l'architecture de la plateforme. Cette fonction permet de configurer Newton
// pour qu'il soit plus performant sur un certain type d'architecture. Les valeurs
// possibles sont 0, 1, et 2 (valeur par dfaut). La valeur 0 utilisera les fonctionnalits
// x87 du processeur, la valeur 1 les instructions SSE et la valeur 2 est gale  la version
// 1 (pour le moment, dans la prochaine version de Newton, 2 devrait permettre d'optimiser
// les programmes pour les processeurs multi-coeurs)
void NewtWorld::NewtSetPlatformArchitecture (int mode) const
{
	// On vrifie que la variable ne soit pas hors des limites
	if (mode < 0 || mode > 2)
		mode = 0;
	
	// Puis on appelle la fonction correspondante
	NewtonSetPlatformArchitecture (_nWorld, mode);
}

// Permet de dfinir le frameRate minimum  laquelle la simulation peut s'effectuer.
// La valeur peut s'talonner entre 60 et 1000 fps (la valeur par dfaut est 60 fps).
// Concrtement, si votre framerate est bloqu  80 fps (comme c'est le cas pour nous ici),
// vous devrez spcifier une valeur de 80 fps. A chaque fois que le framerate descendra
// en dessous de cette limite de 80 fps, Newton effectuera des tapes intermediaires
// lors de la mise  jour des objets pour pouvoir atteindre ce framerate.
void NewtWorld::NewtSetMinimumFramerate (const float frameRate) const
{
	NewtonSetMinimumFrameRate (_nWorld, frameRate);
}

// J'avoue ne pas bien saisir l'intrt de cette fonction, mais je la met quand mme
// ici puisqu'elle fait partie du SDK, donc elle est peut-tre utile  certains ^^. En
// fait, d'aprs mes tests, elle retourne tout simplement la valeur que l'on passe 
// la fonction NewtonUpdate. Ainsi, dans ce programme, je lui passe  chaque fois
// la valeur (1.0f/60.0f). NewtGetTimeStep me retournera donc 0.0166667. N'hsitez pas
//  me contacter si vous savez  quoi a peut servir :).
float NewtWorld::NewtGetTimeStep () const
{
	return (NewtonGetTimeStep (_nWorld));
}

// Retourne la version de Newton sous la forme d'un entier (par exemple 153)
int NewtWorld::NewtGetVersion () const
{
	return (NewtonWorldGetVersion (_nWorld));
}

// Avance la simulation du temps indiqu en paramtre
void NewtWorld::NewtUpdate (const float timeStep) const
{
	// On appelle la fonction NewtonUpdate correspondante
	NewtonUpdate (_nWorld, timeStep);
}

// Cette fonction permet de dfinir la taille du monde Newton. Par dfaut, cette taille
// est de -100;-100;-100  100;100;100, ce qui signifie que tous les objets en dehors
// de cette zone seront automatiquement supprims de la liste active des objets  simuler.
// Pour contrer  cette limitation, libre  vous de dfinir une zone plus grande. Attention
// toutefois  ne pas mettre un champs trop grand, sous peine de faire ralentir fortement
// le moteur
void NewtWorld::NewtSetWorldSize (const CVector & min, const CVector & max) const
{
	NewtonSetWorldSize (_nWorld, &min.x, &max.x);
}

// Permet de mettre un corps hors de la liste des objets  simuler. Comme cela est si bien
// dit dans la doc, cette fonction peut-tre particulirement utile en cas d'optimisation,
// pour mettre un corps hors de la liste des objets actifs.
void NewtWorld::NewtWorldFreezeBody (const NewtonBody * nBody) const
{
	// On appelle la fonction correspondante
	NewtonWorldFreezeBody (_nWorld, nBody);
}

// Permet de dire l'inverse, donc de remettre dans la liste active un objet
void NewtWorld::NewtWorldUnfreezeBody (const NewtonBody * nBody) const
{
	// On appelle la fonction correspondante
	NewtonWorldUnfreezeBody (_nWorld, nBody);
}

// Fonction debug, permet de montrer la gometrie des primitives de collision
void NewtWorld::DebugShowCollision ()
{
	int i;

	glDisable (GL_TEXTURE_2D);

	glBegin (GL_LINES);
	   glColor3f (1.0f, 1.0f, 0.0f);
	   NewtonWorldForEachBodyDo (_nWorld, DebugShowBodyCollision);

	   glColor3f (0.0f, 0.0f, 1.0f);

	   for (i = 0; i < debugCount; ++i) 
	   {
		   glVertex3f (debugFace[i][0].x, debugFace[i][0].y, debugFace[i][0].z);
		   glVertex3f (debugFace[i][1].x, debugFace[i][1].y, debugFace[i][1].z);
	   }
	glEnd();

	glEnable (GL_TEXTURE_2D);
}

// Permet d'afficher la gometrie de la primitive de collision
void DebugShowGeometryCollision (const NewtonBody* body, int vertexCount, const dFloat* faceVertec, int id)
{
	int i;

	i = vertexCount - 1;

	CVector p0 (faceVertec[i * 3 + 0], faceVertec[i * 3 + 1], faceVertec[i * 3 + 2]);

	for (i = 0; i < vertexCount; i ++) 
	{
		CVector p1 (faceVertec[i * 3 + 0], faceVertec[i * 3 + 1], faceVertec[i * 3 + 2]);
		glVertex3f (p0.x, p0.y, p0.z);
		glVertex3f (p1.x, p1.y, p1.z);
 		p0 = p1;
	}
}

// Affiche la gometrie de la primitive de collision du corps pass en paramtre
void DebugShowBodyCollision (const NewtonBody* body)
{
	NewtonBodyForEachPolygonDo (body, DebugShowGeometryCollision);
}

