/// Singleton reprsentant un monde Newton. Cette classe encapsulera toutes les fonctions
/// proposes par Newton Game Dynamics pour grer le monde Newton.

#ifndef NEWTWORLD_HPP
#define NEWTWORLD_HPP

#include <iostream>
#include <GL/glew.h>
#include <Newton.h>

#include "Vector.hpp"

// Prcision du solver
enum SOLVER_MODEL {SOLVER_EXACT = 0, SOLVER_ADAPTIVE};

// Prcision des frictions
enum FRICTION_MODEL {FRICTION_EXACT = 0, FRICTION_ADAPTIVE};

class NewtWorld
{
	private:
		// Constructeur / Destructeur
		NewtWorld ();
		~NewtWorld ();

	public:
		// Retourne l'instance unique de la classe
		static NewtWorld * GetInstance ();

		// Dtruit l'instance unique de la classe
		static void Destroy ();
		
		// Retourne le pointeur vers le monde Newton
		inline NewtonWorld * GetNewtonWorld () const;

		/// --------------------------------------------------------------------------
		/// FONCTION DEBUG
		/// --------------------------------------------------------------------------
		// Permet d'afficher les primitives de collision, et de voir ce que Newton
		// voit en interne
		void DebugShowCollision ();

		/// --------------------------------------------------------------------------
		/// FONCTIONS RELATIVES A NEWTON GAME DYNAMICS
		/// --------------------------------------------------------------------------
		// Rgle la prcision du solver (agit sur les performances)
		void NewtSetSolverModel (SOLVER_MODEL) const;

		// Rgle la prcision du systme de friction (agit sur les performances)
		void NewtSetFrictionModel (FRICTION_MODEL) const;
	
		// Rgle l'architecture de la plateforme (agit sur les performances)
		void NewtSetPlatformArchitecture (int mode) const;

		//
		void NewtSetMinimumFramerate (const float frameRate) const;

		// Renvoie la valeur passe en argument  NewtonUpdate
		float NewtGetTimeStep () const;
		
		// Renvoie la version de Newton Game Dynamics utilise
		int NewtGetVersion () const;
		
		// Met  jour la scne
		void NewtUpdate (const float timeStep) const;

		// Permet de dfinir la taille du monde Newton
		void NewtSetWorldSize (const CVector & min, const CVector & max) const;
		
		// Permet de mettre un corps  "l'arrt"
		void NewtWorldFreezeBody (const NewtonBody * nBody) const;

		// Fait l'inverse :)
		void NewtWorldUnfreezeBody (const NewtonBody * nBody) const;

	private:
		// L'instance unique de la classe
		static NewtWorld * _instance; 
		
		// Notre monde Newton
		NewtonWorld * _nWorld; 

		// Deux variables ncessaires pour le deboggage
		CVector debugFace [1024][2];
		int debugCount;
};

// Fonction inline qui retourne le monde Newton
NewtonWorld * NewtWorld::GetNewtonWorld () const
{
	return _nWorld;
}

#endif // NEWTWORLD.HPP
